// set the dimensions and margins of the graph
const width = 800;
const height = 500;

var curLoc = 'global';

// append the svg object to the body of the page
const svg = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width / 2)
  .attr('height', height);

const svg2 = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width / 2)
  .attr('height', height);

const locs = ['USA', 'Brazil', 'India'];

export const dropdown = d3.select('#locSelect');

dropdown
  .selectAll('[empty]')
  .data(['Global', ...locs])
  .join('option')
  .attr('value', (d) => d)
  .text((d) => d);

dropdown.on('change', function () {
  curLoc = d3.select(this).property('value');
  renderData(0);
});

function render(mapData, countryData) {
  var test = [...mapData.features].filter(
    (c) => c.properties.name === 'usa',
  );

  var projection = d3
    .geoEqualEarth()
    .fitSize([width / 2, height], mapData);
  var path = d3.geoPath(projection);

  function getCountryColor(name) {
    return 'grey';
  }
  const yAttr = 'Number of speakers';

  const plotColorAttr = 'Degree of endangerment';
  const plotColorScale = d3
    .scaleOrdinal()
    .domain([
      'Vulnerable',
      'Definitely endangered',
      'Severely endangered',
      'Critically endangered',
      'Extinct',
    ])
    .range(['orange', 'red', 'maroon', 'purple', 'gray']);
  function getPointColor(d) {
    return plotColorScale(d[plotColorAttr]);
  }

  svg
    .append('g')
    .selectAll('path')
    .data(mapData.features)
    .join('path')
    // draw each country
    .attr('fill', function (d) {
      return getCountryColor(d.properties.name);
    })
    .attr('id', (d) => d.properties.name)
    .attr('d', path);

  const mapMarks = svg
    .selectAll('Mycircle')
    .data(countryData.values())

    .join('circle')
    .attr('id', (d) => d['Name in English'])
    .attr('cx', function (d) {
      return projection([d['Longitude'], d['Latitude']])[0];
    })
    .attr('cy', function (d) {
      return projection([d['Longitude'], d['Latitude']])[1];
    })
    .attr('fill', getPointColor)
    .style('opacity', 0.4)
    .attr('r', 2);

  const y = d3
    .scaleLog()
    .domain(
      d3.extent(
        d3.map(
          countryData
            .values()
            .filter(
              (d) => d[yAttr] > 0 && d[yAttr] != 'NaN',
            ),
          (d) => d[yAttr],
        ),
      ),
    )
    .range([height - 50, 0]);

  svg2
    .append('g')
    .attr('transform', `translate(0,${height - 50})`) // This controls the vertical position of the Axis
    .call(d3.axisBottom(x));

  svg2
    .append('g')
    .attr('transform', `translate(0,0)`) // This controls the vertical position of the Axis
    .call(d3.axisRight(y));

  const scatterMarks = svg2
    .append('g')
    .selectAll('circle')
    .data(countryData.values())
    .join('circle')
    .attr('cx', (d) => d['TrueX'])
    .attr('cy', (d) => y(d[yAttr]))
    .attr('r', 2)
    .attr('class', 'plot')
    .attr('fill', getPointColor)
    .attr('id', (d) => d['Name in English'])
    .style('opacity', 0.3)
    .on('mouseover', function (event, data) {
      console.log(this.id);
      //d3.select(this).attr('fill', 'red');
      highlightMarks(this.id);
    })
    .on('mouseleave', function (event, data) {
      d3.selectAll(`#${this.id}`)
        .attr('fill', getPointColor)
        .attr('r', 2)
        .style('opacity', 0.3);
    });
  scatterMarks
    .filter(function (d) {
      return (
        d['Number of speakers'] == 1 &&
        d['Degree of endangerment'] != 'Extinct'
      );
    })
    .remove();
  const brush = d3.brush().on('start brush end', brushed);

  svg2
    .append('g')
    .call(brush)
    .call(brush.move, [
      [100, 100],
      [200, 200],
    ]);

  function brushed({ selection }) {
    const [[x0, y0], [x1, y1]] = selection;
    const brushedNodes = scatterMarks.filter(
      (d) =>
        // helper function that returns true if the point is
        // within the selection rect

        x0 <= d['TrueX'] &&
        d['TrueX'] <= x1 &&
        y0 <= y(d['Number of speakers']) &&
        y(d['Number of speakers']) <= y1,
    );
    scatterMarks.each((d) =>
      resetMark(d['Name in English']),
    );
    brushedNodes.each((d) =>
      highlightMarks(d['Name in English']),
    );
  }

  function resetMark(id) {
    d3.selectAll(`#${id}`)
      .attr('stroke', null)
      .attr('r', 2)
      .style('opacity', 0.3);
  }

  function highlightMarks(id) {
    // Select everything with this id (polygon and dot), and set the fill
    // color for both
    d3.selectAll(`#${id}`)
      .attr('r', 2)
      .attr('stroke', 'blue')
      .style('opacity', 1)
      .raise();
  }
}

//Read the data
const mapUrl =
  'https://raw.githubusercontent.com/holtzy/D3-graph-gallery/master/DATA/world.geojson';
const countryStatsUrl =
  'https://raw.githubusercontent.com/Dastair/languageData/main/data.csv';
const continentUrl =
  'https://raw.githubusercontent.com/samayo/country-json/master/src/country-by-continent.json';

function normalizeCountryName(s) {
  return s
    .replaceAll(' ', '_')
    .replaceAll('(', '_')
    .replaceAll(')', '_')
    .replaceAll("'", '_')
    .replaceAll('/', '_')
    .replaceAll('?', '_')
    .replaceAll('|', '_')
    .replaceAll('!', '_')
    .toLowerCase();
}
var jitterWidth = 40;

const x = d3
  .scaleBand()
  .domain([
    'Vulnerable',
    'Definitely endangered',
    'Severely endangered',
    'Critically endangered',
    'Extinct',
  ])
  .range([0, width / 2])

  .padding(0.05);

Promise.all([
  d3.json(mapUrl),
  d3.csv(countryStatsUrl),
  d3.json(continentUrl),
]).then(([mapData, csvData, continentData]) => {
  var countryData = new Map();
  csvData.forEach((row) => {
    row['Name in English'] =
      normalizeCountryName(row['Name in English']) + row.ID;
    row['Number of speakers'] =
      row['Number of speakers'] + 1;

    row['TrueX'] =
      x(row['Degree of endangerment']) +
      x.bandwidth() / 1.5 -
      Math.random() * jitterWidth;

    countryData.set(row.ID, row);
  });
  mapData.features.forEach((f) => {
    f.properties.name = normalizeCountryName(
      f.properties.name,
    );
  });

  render(mapData, countryData);
});
